# ==========================================================
# Copyright (C) 2005 Z80-OOLONG. (NAKATSUKA, Yukitaka)
# ==========================================================

require "webrick"
require "base64"

############
# a fix and extension code of WEBrick module. ;-(

module WEBrick
	module HTTPAuth
		class BasicAuth
			def decode64(str); Base64::decode64(str); end
		end
	end

	class HTTPServer
		############
		# Before calling method HTTPServer#mount,
		# hook method servlet.mounted is called
		# if servlet class object respond to it.

		alias __mount__ mount
		def mount(dir, servlet, *options)
			if servlet.respond_to?(:mounted) then
				dir, *options = servlet.mounted(dir, *options)
			end
			__mount__(dir, servlet, *options)
		end
	end

	############
	# class WEBrick::Output an object for output to WEBrick::Response.

	class Output
		def initialize(response)
			@response = response; @response.body = ""; @headbuf = ""
		end

		def <<(lines)
			lines.to_s.each_line("\n") do |line|
				if @headbuf && (/^\r*$/ =~ line) then
					begin
						# for NPH.
						@headbuf.gsub!(%r!^HTTP/1.[01] (\d+) ([^\n]*)\n!m) do
							@response.status = $1.to_i; ""
						end
						headers = WEBrick::HTTPUtils::parse_header(@headbuf)
						if /^(\d+)/ =~ headers['status'][0] then
							@response.status = $1.to_i; headers.delete('status')
						end
						if headers['set-cookie'] then
							@response.cookies.concat(headers['set-cookie'])
							headers.delete('set-cookie')
						end
						headers.each{|k,v| @response[k] = v.join(", ")}
					rescue => e
						raise WEBrick::HTTPStatus::InternalServerError, e.message
					end
					@headbuf = nil
				else
					(@headbuf || @response.body) << line
				end
			end
		end

		def print(*args)
			self << args.map {|arg| arg.to_s}.join
		end
	end

	class HTTPRequest
		def to_cgi
			cgi = ::CGI.new("empty")
			cgi.params.clear
			query.each do |k, v|
				vary = Array::new
				v.each_data do |s|
					if (HTTPUtils::FormData === s) && (filename = s.filename) then
						conttype = if s["content-type"].empty? then
							HTTPUtils::mime_type(filename, HTTPUtils::DefaultMimeTypes)
						else
							s["content-type"]
						end
						vary << ::StringIO.new(s.to_s, "r", filename, conttype)
					else
						vary << s.to_s.gsub(/\r\n/, "\n")
					end
				end
				cgi.params[k] = vary
			end
			cgi.cookies.clear
			cgi.cookies.update(::CGI::Cookie::parse(cookies.map{|c| c.to_s}.join("; ")))
			cgi.set_env_table(self.meta_vars)

			return cgi
		end
	end

	class HTTPResponse
		def to_out; Output::new(self); end
	end
end

# ==========================================================
