module Generator
	class Xml
		ENCODING = {'u' => 'utf-8', 's' => 'Shift_JIS', 'e' => 'euc-jp'}
		ESCAPES = {'<'=>'&lt;','>'=>'&gt;','&'=>'&amp;','"'=>'&quot;','\''=>'&apos;'}
		ENTREFS = [
		    "nbsp",     "iexcl",    "cent",     "pound",    
		    "curren",   "yen",      "brvbar",   "sect",     
		    "uml",      "copy",     "ordf",     "laquo",    
		    "not",      "shy",      "reg",      "macr",     
		    "deg",      "plusmn",   "acute",    "micro",    
		    "para",     "middot",   "cedil",    "ordm",     
		    "raquo",    "iquest",   "Agrave",   "Aacute",   
		    "Acirc",    "Atilde",   "Auml",     "Aring",    
		    "AElig",    "Ccedil",   "Egrave",   "Eacute",   
		    "Ecirc",    "Euml",     "Igrave",   "Iacute",   
		    "Icirc",    "Iuml",     "ETH",      "Ntilde",   
		    "Ograve",   "Oacute",   "Ocirc",    "Otilde",   
		    "Ouml",     "times",    "Oslash",   "Ugrave",   
		    "Uacute",   "Ucirc",    "Uuml",     "Yacute",   
		    "THORN",    "szlig",    "agrave",   "aacute",   
		    "acirc",    "atilde",   "auml",     "aring",    
		    "aelig",    "ccedil",   "egrave",   "eacute",   
		    "ecirc",    "euml",     "igrave",   "iacute",   
		    "icirc",    "iuml",     "eth",      "ntilde",   
		    "ograve",   "oacute",   "ocirc",    "otilde",   
		    "ouml",     "divide",   "oslash",   "ugrave",   
		    "uacute",   "ucirc",    "uuml",     "yacute",   
		    "thorn",    "yuml",     "quot",     "lt",       
		    "gt",       "amp",      "apos",     "OElig",    
		    "oelig",    "Scaron",   "scaron",   "Yuml",     
		    "circ",     "tilde",    "ensp",     "emsp",     
		    "thinsp",   "zwnj",     "zwj",      "lrm",      
		    "rlm",      "ndash",    "mdash",    "lsquo",    
		    "rsquo",    "sbquo",    "ldquo",    "rdquo",    
		    "bdquo",    "dagger",   "Dagger",   "permil",   
		    "lsaquo",   "rsaquo",   "euro",     "Alpha",    
		    "Beta",     "Gamma",    "Delta",    "Epsilon",  
		    "Zeta",     "Eta",      "Theta",    "Iota",     
		    "Kappa",    "Lambda",   "Mu",       "Nu",       
		    "Xi",       "Omicron",  "Pi",       "Rho",      
		    "Sigma",    "Tau",      "Upsilon",  "Phi",      
		    "Chi",      "Psi",      "Omega",    "alpha",    
		    "beta",     "gamma",    "delta",    "epsilon",  
		    "zeta",     "eta",      "theta",    "iota",     
		    "kappa",    "lambda",   "mu",       "nu",       
		    "xi",       "omicron",  "pi",       "rho",      
		    "sigmaf",   "sigma",    "tau",      "upsilon",  
		    "phi",      "chi",      "psi",      "omega",    
		    "thetasym", "upsih",    "piv",      "bull",     
		    "hellip",   "prime",    "Prime",    "oline",    
		    "frasl",    "weierp",   "image",    "real",     
		    "trade",    "alefsym",  "larr",     "uarr",     
		    "rarr",     "darr",     "harr",     "crarr",    
		    "lArr",     "uArr",     "rArr",     "dArr",     
		    "hArr",     "forall",   "part",     "exist",    
		    "empty",    "nabla",    "isin",     "notin",    
		    "ni",       "prod",     "sum",      "minus",    
		    "lowast",   "radic",    "prop",     "infin",    
		    "ang",      "and",      "or",       "cap",      
		    "cup",      "int",      "sim",      "cong",     
		    "asymp",    "ne",       "equiv",    "ge",       
		    "sub",      "sup",      "nsub",     "sube",     
		    "supe",     "oplus",    "otimes",   "perp",     
		    "sdot",     "lceil",    "rceil",    "lfloor",   
		    "rfloor",   "lang",     "rang",     "loz",      
		    "spades",   "clubs",    "hearts",   "diams",    
		    "#[0-9]+",  "#[Xx][0-9a-fA-F]+"
		]

		def initialize
			@escapes = self.class::ESCAPES unless defined?(@escapes)
			@entrefs = self.class::ENTREFS unless defined?(@entrefs)
			esckeys = @escapes.keys
			esckeys[esckeys.index("&")] = "&(?!(?:#{@entrefs.join('|')});)"
			@re_escapes = /(#{esckeys.join('|')})/
			@unescapes = @escapes.invert
			@re_unescapes = /(#{@unescapes.keys.join('|')})/
		end
		def generate(node, outobj='')
			case node
			when Element then
				if node.hasChildNodes then
					list = node.childNodes
					outobj << begin_tag(node)
					0.upto(list.length-1) do |i|
						generate(list.item(i), outobj)
					end
					outobj << end_tag(node)
				else
					outobj << empty_tag(node)
				end
			when Text then
				outobj << escape(node.nodeValue)
			when Comment then
				outobj << '<!--'+escape(node.nodeValue)+'-->'
			when Document then
				encode = ENCODING[$KCODE[0].chr.downcase]
				outobj << %Q[<?xml version="1.0" encoding="#{encode}"?>\n]
				generate(node.documentElement, outobj)
			when DocumentFragment then
				list = node.childNodes
				0.upto(list.length-1) do |i|
					generate(list.item(i), outobj)
				end
			end
			return outobj
		end
	private
		def begin_tag(node)
			tag = "<#{node.nodeName}"
			map = node.attributes
			if map then
				0.upto(map.length-1) do |i|
					attr = map.item(i)
					tag << %Q[ #{attr.nodeName}="#{escape(attr.nodeValue)}"]
				end
			end
			tag << '>'
			return tag
		end
		def end_tag(node)
			return "</#{node.nodeName}>"
		end
		def empty_tag(node)
			return begin_tag(node).sub(/>$/, '/>')
		end
		def escape(text)
			text.gsub(@re_escapes){@escapes[$1]}
		end
		def unescape(text)
			text.gsub(@re_unescapes){@unescapes[$1]}
		end
	end
end
