require 'vikiwikiplugin'

module VikiWiki
	module Plugins
		module TbUtil
			DEFAULT_TB_CGI  = "/cgi-bin/tb.cgi"

			def tb_uri(page = @sys.page, static = @sys.static?)
				staticuri = (@sys["TB_CONF"] || {})["STATICURI"]
				baseuri = (@sys["TB_CONF"] || {})["BASEURI"]

				if static then
					uri = StaticHTML::new(@sys, page.name).uri(@id)
					staticuri && uri.gsub!(/#{@sys["STATICURI"]}/, staticuri)
				else
					uri = page.uri(nil, @id)
					baseuri && uri.gsub!(/#{@sys["BASEURI"]}/, baseuri)
				end
				uri.gsub!(%r|/+|, "/"); return uri
			end
		end

		class Tbrecv
			include BaseModule
			include TbUtil

			LBL = Hash::new
			LBL["PRMLNK_NAM"] = "Parma Link: "
			LBL["PINGURL_NAM"] = "Trackback Ping URL: "

			def ondesc; <<DSC; end
The plugin can receive a trackback ping.
DSC
			def safe; 4; end

			def onpost
				pos = @prms[0]; fmt = @prms[1]

				title = @sys.cgi.params["tbrecvtitle"][0].to_s
				url = @sys.cgi.params["tbrecvurl"][0].to_s
				excerpt = @sys.cgi.params["tbrecvexcerpt"][0] || "no excerpt"
				blog_name = @sys.cgi.params["tbrecvblog_name"][0] || "no blog_name"

				title.gsub!(/\n/, ""); url.gsub!(/\n/, "")
				excerpt.gsub!(/\n/, ""); blog_name.gsub!(/\n/, "")

				insert(pos) do
					fmt.replace_words({
						"title" => shorten(title, 252), "url" => url,
						"blog_name" => shorten(blog_name, 252),
						"excerpt" => shorten(excerpt, 252),
						"date" => Time.now.strftime("%Y/%m/%d"),
						"time" => Time.now.strftime("%H:%M:%S")
					})
				end
			end

			def onview
				tbcgi = @sys["TB_CGI"] || DEFAULT_TB_CGI
				host = (@sys["TB_CONF"] || {})["HOSTNAME"] || @sys.cgi.host
				parmalink = "http://#{host}#{tb_uri}"
				tbpingurl = "http://#{host}#{tbcgi}/#{@sys.page.name}"
				tbpingurl << ";#{@id}" if @id

				return (<<EOTEXT)
<div class="tbrecv">
<table summary="Trackback Ping URL">
<tr><td>#{LBL["PRMLNK_NAM"]}</td><td>#{parmalink}</td></tr>
<tr><td>#{LBL["PINGURL_NAM"]}</td><td>#{tbpingurl}</td></tr>
</table>
<!--
<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         xmlns:dc="http://purl.org/dc/elements/1.1/"
         xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
<rdf:Description
    rdf:about="#{parmalink}"
    dc:identifier="#{parmalink}"
    dc:title="#{@sys.title}"
    trackback:ping="#{tbpingurl}" />
</rdf:RDF>
-->
</div>
EOTEXT
			end

			private

			def shorten(str, len)
				str = toeuc(str)[0,len]
				(str.count("\x80-\xff") % 2 == 0) || str.gsub!(/[\x80-\xff]$/, " ")
				(str.length < len) || (str << "...")
				return tosyscode(str)
			end

			def toeuc(str)
				encoding = @sys.encoding
				cnv = USEconv::cnvmethod(encoding, "euc-jp")
				raise "Conversion Failed #{encoding} => euc-jp" if !cnv
				return cnv.call(str)
			end

			def tosyscode(str)
				encoding = @sys.encoding
				cnv = USEconv::cnvmethod("euc-jp", encoding)
				raise "Conversion Failed euc-jp => #{encoding}" if !cnv
				return cnv.call(str)
			end
		end
	end
end
