require 'vikiwikiplugin'
require 'uri'
require 'net/http'

require 'plugin/tbrecv'

module VikiWiki
	module Plugins
		class Tbsend
			include BaseModule
			include TbUtil

			LBL = Hash::new
			LBL['SPINGURL_NAM'] = "Trackback Ping URL"
			LBL['TITLE_NAM'] = "Title"
			LBL['URL_NAM'] = "URL"
			LBL['BNAME_NAM'] = "Blog Name"
			LBL['EXCPT_NAM'] = "Excerpt"
			LBL['TBSEND_NAM'] = "Send Trackback Ping"
			MSG = Hash::new
			MSG['SUCCESS'] = "Succeed to send a trackback ping."
			MSG['FAILED'] = "Failed to send a trackback ping."
			MSG['ADMINONLY'] = "This page can be used only by the administrator."

			def ondesc; <<DSC; end
The plugin can send a trackback ping.
DSC
			def safe; 2; end

			def onpost
				params = @sys.cgi.params

				tbpingurl = params["tbsendpingurl"][0].to_s
				title = toutf8(params["tbsendtitle"][0].to_s)
				url = params["tbsendurl"][0].to_s
				blog_name = toutf8(params["tbsendblog_name"][0].to_s)
				excerpt = toutf8(params["tbsendexcerpt"][0].to_s)

				if !tbpingurl.empty? && !url.empty? then
					query = "url=#{::CGI::escapePATH(url)}"
					title.empty? || (query << "&title=#{::CGI::escapePATH(title)}")
					excerpt.empty? || (query << "&excerpt=#{::CGI::escapePATH(excerpt)}")
					blog_name.empty? || \
						(query << "&blog_name=#{::CGI::escapePATH(blog_name)}")

					tbpinguri = ::URI.parse(tbpingurl.untaint)
					::Net::HTTP.version_1_2
					::Net::HTTP.start(tbpinguri.host, tbpinguri.port) do |http|
						response = http.post(tbpinguri.path, query, {
							'Content-type' => "application/x-www-form-urlencoded"
						})
						result = response.body; result.gsub!(/\n/, "")
						if %r[<error>.*0.*</error>] === result then
							@result = "<p><b>#{MSG['SUCCESS']}</b></p>"
						else
							message = "(No message)"
							if !(/^2[0-9][0-9]/ === response.code) then
								message = "#{response.code} #{response.message}"
							elsif %r[<error>.*1.*</error>] === result then
								(%r[<message>(.*)</message>] === result) && (message = $1)
							end
							@result = "<p><b>#{MSG['FAILED']}</b> -- #{message}</p>\n<!--\n#{result}\n-->\n"
						end
					end
				end
			end

			def onview
				return "" if @sys.static?
				raise MSG['ADMINONLY'] if (@sys.user != @sys['ADMIN'])

				title = @sys.page.alias
				url  =	case @prms[0].to_s
					when "static"; tb_uri(@sys.page, true)
					when "dynamic", ""; tb_uri(@sys.page, false)
					else; @prms[0].to_s
					end
				hostname = (@sys["TB_CONF"] || {})["HOSTNAME"]
				url = "http://#{hostname || @sys.cgi.host}#{url}"
				blog_name = (@sys["TB_CONF"] || {})["BLOG_NAME"].to_s
				excerpt = @prms[1].to_s

				ctls = Array::new
				ctls << [LBL['SPINGURL_NAM'], text('tbsendpingurl', "", 40)]
				ctls << [LBL['TITLE_NAM'], text('tbsendtitle', title, 40)]
				ctls << [LBL['URL_NAM'], text('tbsendurl', url, 40)]
				ctls << [LBL['BNAME_NAM'], text('tbsendblog_name', blog_name, 40)]
				ctls << [LBL['EXCPT_NAM'], textarea('tbsendexcerpt', excerpt, 60, 5)]
				ctls << [hidden('tbsend', 'yes')]
				ctls << [submit('exec', LBL['TBSEND_NAM'])]

				return (<<EOTEXT)
<div class="tbsend">
#{@result}
#{plugin_form {ctls}}
</div>
EOTEXT
			end

			private

			def toutf8(str)
				encoding = @sys.encoding
				cnv = USEconv::cnvmethod(encoding, "utf-8")
				raise "Conversion Failed #{encoding} => utf-8" if !cnv
				return cnv.call(str)
			end
		end
	end
end
