# ==========================================================
# Copyright (C) 2006 Z80-OOLONG. (NAKATSUKA, Yukitaka)
# ==========================================================

require 'generator/xml'
begin
	require 'tidy'
	raise if !$LIBTIDY_SO_PATH
	raise if (File.stat($LIBTIDY_SO_PATH).mode & 0422 != 0400)
	if Tidy.path != $LIBTIDY_SO_PATH then
		Tidy.path = $LIBTIDY_SO_PATH
	end
	$TIDY_INSTALLED = true
rescue LoadError, StandardError
	$TIDY_INSTALLED = false
end

module VikiWiki
	class Tidy
		ENTREF_PATTERN =
			/&((?:#{Generator::Xml::ENTREFS.join("|")});)/mn
		ENCODE_OPTION = {
			"e" => "raw", "s" => "shiftjis", "u" => "utf8"
		}
		class Error < StandardError; end

		def initialize(sys)
			@sys = sys; @dynamic = @static = false
			@options = nil
			return unless $TIDY_INSTALLED
			@dynamic = (/^(all|dynamic)$/ === @sys["TIDY_MODE"])
			@static = (/^(all|static)$/ === @sys["TIDY_MODE"])
			@options = @sys["TIDY_OPTION"] || {}
			@options = @options.inject({}) do |opts, (opt, val)|
				if /[a-z0-9\-_]+$/ === opt then
					opts[sanitize(opt)] = sanitize(val)
				end; opts
			end
			@options["show-warnings"] ||= "no"
		end
		def clean(text)
			return text unless @options
			return text if (@sys.static? && !@static)
			return text if (!@sys.static? && !@dynamic)
			@options["input-xml"] = (@sys.content_type == XML)
			@options["char-encoding"] = ENCODE_OPTION[@sys.code]
			::Tidy.open(@options) do |tidyobj|
				text.untaint; otext = text
				text = text.gsub(ENTREF_PATTERN, '&amp;\1')
				text = tidyobj.clean(text)
				text.gsub!("&amp;", "&")
				unless tidyobj.errors.empty? then
					tidyerr = Error.new("Tidy error\n#{tidyobj.errors.join('')}")
					if @sys.static? || (@sys.content_type == XML) then
						raise tidyerr
					else
						errbody = @sys.error_onview(tidyerr, tidyerr.backtrace)
						text = otext.sub(/(<body[^>]*>)/mn, "\\1\n#{errbody}")
					end
				end
			end
			return text
		end

		private

		def sanitize(value)
			value = value.to_s
			if /[|&;`$<>]/ =~ value then
				raise Error, "Sanitize insecure error."
			end
			value.untaint; return value
		end
	end
end

# ==========================================================
