# ==========================================================
# Copyright (C) 2005 Z80-OOLONG. (NAKATSUKA, Yukitaka)
# ==========================================================

require 'vikiwikisys'

module VikiWiki
	class TrackBack
		def initialize(file, conf, cgi, methodcheck = true)
			begin
				if methodcheck && cgi.request_method != "POST" then
					raise "unsupported method `#{cgi.request_method}'."
				end

				tbpage = tbid = nil
				if %r[/([^;]*)(;(.*))?$] === cgi.path_info then
					tbpage = $1; tbid = $3
				end

				title = cgi.params["title"]; excerpt = cgi.params["excerpt"]
				url = cgi.params["url"]; blog_name = cgi.params["blog_name"]
				raise "url parameter is not existed." unless url[0]
				title[0] || (title = url.dup)

				cgi.params.clear
				sys = VikiWiki::Sys.new(file, conf, cgi); sys.setup
				node = sys.pages[tbpage].node

				if (params = search_tbrecv_plugin_node(node, tbid)) then
					cgi.params.update({
						"c" => [sys.code], "l" => [sys.lang],
						"f" => [PluginsPage], "p" => [tbpage],
						"tbrecvtitle" => title, "tbrecvurl" => url
					})

					title[0] = tocode(sys.encoding, title[0])
					if excerpt[0] then
						excerpt[0] = tocode(sys.encoding, excerpt[0])
						cgi.params["tbrecvexcerpt"] = excerpt
					end
					if blog_name[0] then
						blog_name[0] = tocode(sys.encoding, blog_name[0])
						cgi.params["tbrecvblog_name"] = blog_name
					end
					cgi.params.update(params)

					sys = VikiWiki::Sys.new(file, conf, cgi); sys.setup
					sys.plugin_onpost

					sys.print \
						%[Content-Type: application/xml; charset=iso-8859-1\n\n] + \
						%[<?xml version="1.0" encoding="iso-8859-1"?>\n] + \
						%[<response>\n] + \
						%[<error>0</error>\n] + \
						%[</response>\n]
				else
					raise "#tbrecv#{tbid && ('#' + tbid)} is not found in #{tbpage}."
				end
			rescue
				btrace = conf["DEBUG"] ? ::CGI::escapeHTML($@.to_a.join("\n")) : ""
				($VIKIWIKIOUT || $>).print \
					%[Content-Type: application/xml; charset=iso-8859-1\n\n] + \
					%[<?xml version="1.0" encoding="iso-8859-1"?>\n] + \
					%[<response>\n] + \
					%[<error>1</error>\n] + \
					%[<message>#{::CGI::escapeHTML($!.to_s)}</message>\n] + \
					%[<!--\n#{btrace}\n-->\n] + \
					%[</response>\n]
			end
		end

		private

		def search_tbrecv_plugin_node(node, id = nil)
			node.each_node("plugin") do |elem|
				nodeattr = elem.getAttributeHash
				children = elem.childNodes
				if (nodeattr["name"] == "tbrecv") && \
				   (nodeattr["id"] == id) then
					params = {"pi_name" => ["tbrecv"]}
					nodeattr["id"] && (params["pi_id"] = [nodeattr["id"]])
					0.upto(children.length - 1) do |i|
						param = children.item(i)
						params["pi_#{i}"] = [param.getTextValues.join("")]
					end
					params["pi_plain"] = [nodeattr["plain"]]
					params["pi_inline"] = [(nodeattr["wiki-node-type"] == "inline").to_s]
					return params
				end
			end
		end

		def tocode(encoding, str)
			guesscode = USEconv::guess(str)
			cnv = USEconv::cnvmethod(guesscode, encoding)
			raise "Conversion Failed #{guesscode} => #{encoding}" if !cnv
			return cnv.call(str)
		end
	end
end

# ==========================================================
